#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#define DIM_H	800
#define DIM_V	600
	
int main()
{
	//le 3 superfici principali
	SDL_Surface *bg;
	SDL_Surface *tank;
	SDL_Surface *screen;
	//superficie temporanea
	SDL_Surface *temp;

	// si inizializza il sistema video 
	if( SDL_Init(SDL_INIT_VIDEO) <0 )
	{
		printf("Errore init SDL: %s\n", SDL_GetError());
	        return 1;
	}

	// all'uscita del programma esegui SDL_Quit per risistemare le cose
	// utilizzando atexit si evita di dover chiamare SDL_QUIT prima di ogni return
	atexit(SDL_Quit);

	//flag per settare lo schermo
	Uint32 flags=SDL_HWSURFACE|SDL_DOUBLEBUF;
	
	// per rendere visualizzabile la Superficie bisogna utilizzare SDL_SetVideoMode
	if((screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags))==NULL)
	{
		  printf("Problemi settaggio video: %s\n", SDL_GetError());
		    return 1;
	}

	//carica sfondo
	if((temp = IMG_Load("sabbia.png"))==NULL)
    	{
		printf("Errore caricamento sfondo: %s\n",IMG_GetError());
		return 1;
	}

	//Adatta l'img dello sfondo al display attuale e l'assegna ad una surface
	bg=SDL_DisplayFormat(temp);

	//si libera temp
	SDL_FreeSurface(temp);

	//copia sfondo su screen
	SDL_BlitSurface(bg, NULL, screen, NULL);

	//carica tank 
	if((temp = IMG_Load("tank.png"))==NULL)
    	{
		printf("Errore caricamento tank: %s\n",IMG_GetError());
		return 1;
	}

	//prendiamo la mappatura del colore #0000FF in base al nostro formato di pixel
	Uint32 colorkey = SDL_MapRGB(temp->format, 0, 0, 0xFF);
	//impostiamo il colore come trasparente e attiviamo l'accellerazione RLE
    	SDL_SetColorKey(temp, SDL_SRCCOLORKEY|SDL_RLEACCEL, colorkey);
	
	//Adatta l'img del tank al display attuale
	tank = SDL_DisplayFormat(temp);
	
	//si libera temp
	SDL_FreeSurface(temp);
    
	//flag fine main loop
	int done=0;
	
	//contatori vari
	int i,num_tank=1;
	int dist=tank->h+50;
	int passo=3;

	//4 flag direzione
	int up,right,down,left;
	//inizialmente non si v da nessuna parte
	up=right=down=left=0;

	//le cordinate del tank
	int xpos=0,ypos=0;

	//rettangolo del tank
    	SDL_Rect rect_tank;
	rect_tank.x=0;
	rect_tank.y=0;
	rect_tank.h=tank->h;
	rect_tank.w=tank->w;
	
	//dichiariamo una struttura SDL_Event
	SDL_Event event;

	//game loop, ovvero il loop che viene eseguito finch non si esce
	while(!done)
	{
		// SDL_PollEvent attiva il gestore di eventi
		while ( SDL_PollEvent(&event) )
		{
			//premendo la x della finestra col mouse si esce
			if ( event.type == SDL_QUIT ) 
				done = 1;

			//viene premuto un tasto
			else if ( event.type == SDL_KEYDOWN )
			{
				switch(event.key.keysym.sym)
				{
					//ma si esce anche premendo Esc
					case SDLK_ESCAPE: 	
						done = 1;
						break;

					//premendo RETURN creiamo un altro tank
					case SDLK_RETURN: 	
						//lo creiamo se c'entra nello schermo
						if(num_tank<DIM_V/dist)
							num_tank++;
						break;

					//premendo spazio si passa dalla modalit schermo 
					//intero alla modalit finestra e viceversa
					case SDLK_SPACE: 	
						flags^=SDL_FULLSCREEN;
						screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags);
						//dobbiamo ridisegnare lo sfondo
						SDL_BlitSurface(bg, NULL, screen, NULL);
						break;

					//FRECCIA SU
					case SDLK_UP: 	
						up=1;
						break;

					//FRECCIA DX
					case SDLK_RIGHT: 	
						right=1;
						break;

					//FRECCIA GIU
					case SDLK_DOWN: 	
						down=1;
						break;

					//FRECCIA SX
					case SDLK_LEFT: 	
						left=1;
						break;
						
					default:
						break;
				}
			}

			//viene rilasciato un tasto
			else if ( event.type == SDL_KEYUP)
			{
				switch(event.key.keysym.sym)
				{
					//FRECCIA SU
					case SDLK_UP: 	
						up=0;
						break;

					//FRECCIA DX
					case SDLK_RIGHT: 	
						right=0;
						break;

					//FRECCIA GIU
					case SDLK_DOWN: 	
						down=0;
						break;

					//FRECCIA SX
					case SDLK_LEFT: 	
						left=0;
						break;

					default:
						break;
				}
			}

		}

		//FRECCIA SU: 
		//se il tank non  posizionato a (x,0) andiamo su
		if(up) 
		{	
			if(ypos)
				ypos-=passo;
		}
		//FRECCIA GIU: 
		//approssima il bordo inferiore in base al numero di tank e a DIM_V
		if(down) 
		{
			if(ypos*num_tank<(DIM_V-tank->h))
				ypos+=passo; 
		}
		//FRECCIA SX: 
		//se il tank non  posizionato a (0,y) andiamo a sx
	    	if(left) 
		{
			if(xpos)
				xpos-=passo;
		}
		//FRECCIA DX: 
		//se il tank non  posizionato a (DIM_H-tank->w,y) andiamo a dx
		if(right) 
		{
			if(xpos<(DIM_H-tank->w))
				xpos+=passo; 
		}

		//si ridisegna lo sfondo per coprire le scie degli spostamenti
		SDL_BlitSurface(bg, NULL, screen, NULL);

		//si aggiorna la posizione di x
		rect_tank.x=xpos;
		//si aggiornano le dimensioni del rettangolo che possono essere 
		//state modificate da Blit precedenti
		rect_tank.h=tank->h;
		rect_tank.w=tank->w;

		for(i=0;i<num_tank;i++)
		{
			//si aggiorna la posizione di y per ogni tank
			rect_tank.y=ypos+(i*dist);
			//si disegna il tank su screen
			SDL_BlitSurface(tank, NULL, screen, &rect_tank);
		}

		//svuota i buffer ed aggiorna lo schermo
		SDL_Flip(screen);

		//si f respirare il processore
		SDL_Delay(1);
	}

	//libera la memoria utilizzata per mantenere le immagini
    	SDL_FreeSurface(screen);
    	SDL_FreeSurface(bg);
	SDL_FreeSurface(tank);

	return 0;
}
